from kutana.kutana import Kutana
from kutana.environments.vk import create_vk_env


def VKKutana(token=None, configuration=None):
    """Shortcut for creating kutana object and applying VK environment."""

    # Create engine
    kutana = Kutana()

    # Create VK controller and environment
    kutana.apply_environment(
        create_vk_env(token=token, configuration=configuration)
    )

    return kutana



from kutana import Kutana, DumpingController, Plugin
from contextlib import contextmanager
import unittest


class KutanaTest(unittest.TestCase):
    def setUp(self):
        self.kutana = None

        self.target = []
        self.actual = []

    @contextmanager
    def dumping_controller(self, queue):
        if self.kutana is None:
            self.kutana = Kutana()

        self.controller = DumpingController(*queue)

        self.kutana.add_controller(self.controller)

        self.plugin = Plugin()

        self.plugins = [self.plugin]

        try:
            yield self.plugin
        finally:
            for plugin in self.plugins:
                self.kutana.executor.register_plugins(plugin)

            self.kutana.run()

        self.assertEqual(self.target, self.actual)



from test_framework import KutanaTest
import logging


logging.disable(logging.CRITICAL)


class TestExecutors(KutanaTest):
    def test_just_dumping(self):
        self.target = ["message"] * 10

        with self.dumping_controller(self.target):

            async def new_update(controller_type, update, env):
                if controller_type == "dumping":
                    self.actual.append(update)

                else:
                    self.assertEqual(controller_type, "kutana")

            self.kutana.executor.register(new_update)

    def test_exception(self):
        self.called = 0

        with self.dumping_controller(["message"] * 10):

            async def new_update(controller_type, update, env):
                if controller_type == "dumping":
                    raise Exception

            self.kutana.executor.register(new_update)

            async def new_error(controller_type, update, env):
                self.called += 1

            self.kutana.executor.register(new_error, error=True)

        self.assertEqual(self.called, 10)

    def test_two_dumping(self):
        self.target = ["message"] * 10

        with self.dumping_controller(self.target):
            with self.dumping_controller(self.target):
                self.target *= 2

                async def new_update(controller_type, update, env):
                    if controller_type == "dumping":
                        self.actual.append(update)

                    else:
                        self.assertEqual(controller_type, "kutana")

                self.kutana.executor.register(new_update)

    def test_two_callbacks_and_two_dumpings(self):
            self.target = ["message"] * 10

            with self.dumping_controller(self.target):
                with self.dumping_controller(self.target):
                    self.target *= 4

                    async def new_update_1(controller_type, update, env):
                        if controller_type == "dumping":
                            self.actual.append(update)

                    async def new_update_2(controller_type, update, env):
                        if controller_type == "dumping":
                            self.actual.append(update)

                    self.kutana.executor.register(new_update_1)
                    self.kutana.executor.register(new_update_2)

    def test_decorate_or_call(self):
        self.target = ["message"] * 10

        with self.dumping_controller(self.target):
            self.target *= 2

            @self.kutana.executor.register()
            async def new_update(controller_type, update, env):
                if controller_type == "dumping":
                    self.actual.append(update)

            self.kutana.executor.register(new_update)



from kutana.logger import logger

from kutana.kutana import *  # lgtm [py/polluting-import]
from kutana.executor import *  # lgtm [py/polluting-import]
from kutana.plugins.plugin import *  # lgtm [py/polluting-import]

from kutana.controllers.vk import *  # lgtm [py/polluting-import]
from kutana.controllers.dumping import *  # lgtm [py/polluting-import]

from kutana.environments.vk import *  # lgtm [py/polluting-import]
from kutana.tools.structures import *  # lgtm [py/polluting-import]
from kutana.tools.functions import *  # lgtm [py/polluting-import]
from kutana.shortcuts import *  # lgtm [py/polluting-import]

name = "kutana"
__version__ = "0.0.2"



from kutana import VKKutana, VKResponse, Executor, load_plugins, objdict, \
    icedict, create_vk_env
import kutana.plugins.converters.vk as converters_vk
import unittest
import asyncio


class TestMiscellaneous(unittest.TestCase):
    def test_objdict(self):
        a = objdict()

        self.assertEqual(str(a), "{}")
        self.assertEqual(len(a), 0)
        self.assertEqual(list(i for i in a), [])

        a.a = 10  # pylint: disable=E0237
        self.assertEqual(a.a, 10)
        self.assertEqual(a["a"], 10)
        self.assertEqual(a.get("a"), 10)

        a["b"] = {"a": 5}
        self.assertEqual(a.b.a, 5)

        a["c"] = [1, 2]
        self.assertIs(a.c, a["c"])

        del a["a"]
        del a["b"]
        del a["c"]

        self.assertEqual(a, {})

    def test_icedict(self):
        a = icedict(a=10)

        self.assertEqual(str(a), "{'a': 10}")
        self.assertEqual(len(a), 1)
        self.assertEqual(list(i for i in a), ["a"])
        self.assertEqual(a["a"], 10)
        self.assertEqual(a.a, 10)

        b = icedict(a=10)

        self.assertEqual(a, b)
        self.assertEqual(a, {"a": 10})

        with self.assertRaises(TypeError):
            a["a"] = 10

    def test_functions(self):
        loaded_plugins = load_plugins("test/test_plugins/")

        self.assertEqual(len(loaded_plugins), 1)
        self.assertEqual(loaded_plugins[0].name, "Memory")

        executor = Executor()
        executor.register_plugins(*loaded_plugins)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(executor("dumping", "message"))

        self.assertEqual(loaded_plugins[0].memory, "message")

    def test_create_vk_env(self):
        create_vk_env(token="token")

        with self.assertRaises(ValueError):
            create_vk_env(configuration="configuration.json.example")

        with self.assertRaises(ValueError):
            create_vk_env(token="")

    def test_vk_shortcut(self):
        vkkutana = VKKutana(token="token")

        self.assertIsNotNone(vkkutana)

    def test_vk_conversation(self):
        arguments = {}

        async def fake_resolveScreenName(*args, **kwargs):
            return VKResponse(False, "", "", {"object_id": 1}, "")

        resolveScreenName = converters_vk.resolveScreenName
        converters_vk.resolveScreenName = fake_resolveScreenName

        loop = asyncio.get_event_loop()

        loop.run_until_complete(
            converters_vk.convert_to_message(
                arguments,
                {"object": {"date": 1, "random_id": 0, "fwd_messages": [],
                "important": False, "peer_id": 1,
                "text": "echo [club1|\u0421\u043e] 123", "attachments": [],
                "conversation_message_id": 1411, "out": 0, "from_id": 1,
                "id": 0, "is_hidden": False}, "group_id": 1,
                "type": "message_new"},
                {},
                {w: 1 for w in ("reply", "send_msg", "request",
                "upload_photo", "upload_doc")}
            )
        )

        self.assertEqual(arguments["message"].text, "echo  123")
        self.assertEqual(arguments["attachments"], ())

        loop.run_until_complete(
            converters_vk.convert_to_message(
                arguments,
                {"object": {"date": 1, "random_id": 0, "fwd_messages": [],
                "important": False, "peer_id": 1,
                "text": "echo [club1|\u0421\u043e] 123", "attachments": [],
                "conversation_message_id": 1411, "out": 0, "from_id": 1,
                "id": 0, "is_hidden": False}, "group_id": 2,
                "type": "message_new"},
                {},
                {w: 1 for w in ("reply", "send_msg", "request",
                "upload_photo", "upload_doc")}
            )
        )

        self.assertEqual(arguments["message"].text, "echo [club1|\u0421\u043e] 123")
        self.assertEqual(arguments["attachments"], ())

        converters_vk.resolveScreenName = resolveScreenName


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
:copyright: (c) 2018 by Michael Krukov
:license: MIT, see LICENSE for more details.
"""

from setuptools.command.install import install
import setuptools
import os
import sys


VERSION = "0.0.2"


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version."""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('TRAVIS_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="kutana",
    version=VERSION,
    author="Michael Krukov",
    author_email="krukov.michael@ya.ru",
    keywords="engine social-networks messengers bots asyncio",
    description="The engine for developing bots for soc. networks, instant messengers and other systems.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/vk-brain/kutana",
    packages=["kutana"],
    install_requires=[
        "requests==2.19",
        "aiohttp==3.3"
    ],
    python_requires='>=3.5',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
    ],
)



from kutana import Kutana, VKController, ExitException, Plugin, \
    logger, create_vk_env
import unittest
import requests
import time
import json
import os


logger.setLevel(40)  # ERROR LEVEL


# Theese tests requires internet and token of VK user account as well as VK group.
# You can set these values through environment variables:
# TEST_TOKEN and TEST_UTOKEN.
#
# Alternatively you can use json file `configuration_test.json` with format like this:
# {
#   "token": "токен группы",
#   "utoken": "токен пользователя. который модет писать в группу"
# }


class TestControllerVk(unittest.TestCase):
    def setUp(self):
        try:
            with open("configuration_test.json") as o:
                self.conf = json.load(o)

        except FileNotFoundError:
            self.conf = {
                "token": os.environ.get("TEST_TOKEN", ""),
                "utoken": os.environ.get("TEST_UTOKEN", ""),
            }

        self.messages_to_delete = set()
        this_case = self

        if not self.conf["token"] or not self.conf["utoken"]:
            self.skipTest("No authorization found for tests.")

        async def create_receiver(self):
            actual_reci = await self.original_create_receiver()

            empty_if_done = [1]

            async def reci():
                if not empty_if_done:
                    raise ExitException

                empty_if_done.pop(0)

                this_case.messages_to_delete.add(
                    str(
                        this_case.ureq(
                            "messages.send",
                            message="echo message",
                            peer_id=-self.group_id
                        )
                    )
                )

                return await actual_reci()

            return reci

        VKController.original_create_receiver = VKController.create_receiver
        VKController.create_receiver = create_receiver

        self.kutana = Kutana()

        self.kutana.apply_environment(
            create_vk_env(token=self.conf["token"])
        )

    def ureq(self, method, **kwargs):
        data = {"v": "5.80"}

        for k, v in kwargs.items():
            if v is not None:
                data[k] = v

        response = requests.post(
            "https://api.vk.com/method/{}?access_token={}".format(
                method,
                self.conf["utoken"]
            ),
            data=data
        ).json()

        time.sleep(0.34)

        return response["response"]

    def tearDown(self):
        VKController.create_receiver = VKController.original_create_receiver

        self.ureq("messages.delete", message_ids=",".join(self.messages_to_delete), delete_for_all=1)

    def test_controller_vk(self):
        plugin = Plugin()

        self.called = False

        async def on_regexp(message, env, **kwargs):
            self.assertEqual(env.match.group(1), "message")
            self.assertEqual(env.match.group(0), "echo message")

            a_image = await env.upload_photo("test/author.png")
            a_audio = await env.upload_doc("test/girl.ogg", doctype="audio_message", filename="file.ogg")

            self.assertTrue(a_image.id)
            self.assertTrue(a_audio.id)

            resp = await env.reply("Спасибо.", attachment=a_image)

            self.assertTrue(resp.response)

            resp = await env.request("messages.delete", message_ids=str(resp.response), delete_for_all=1)

            self.assertTrue(resp.response)

            self.called = True

        plugin.on_regexp_text(r"echo (.+)")(on_regexp)

        self.kutana.executor.register_plugins(plugin)

        self.kutana.run()

        self.assertTrue(self.called)



from kutana.executor import Executor
from kutana.exceptions import ExitException
from kutana.tools.structures import objdict
import asyncio


class Kutana:
    """Main class for constructing engine."""

    def __init__(self, executor=None, loop=None):
        self.controllers = []
        self.executor = executor or Executor()

        self.loop = loop or asyncio.get_event_loop()

        self.running = True
        self.loops = []
        self.tasks = []
        self.gathered_loops = None

        self.settings = objdict()

    def add_controller(self, controller):
        """Adding controller to engine."""

        self.controllers.append(controller)

    def apply_environment(self, environment):
        """Shortcut for adding controller and updating executor."""

        self.add_controller(environment["controller"])
        self.executor.add_callbacks_from(environment["executor"])

    async def shedule_update_process(self, controller_type, update):
        """Shedule update to be processed."""

        await self.ensure_future(self.executor(controller_type, update))

    async def ensure_future(self, awaitable):
        """Shurtcut for asyncio.ensure_loop with curretn loop."""

        await asyncio.ensure_future(awaitable, loop=self.loop)

    async def loop_for_controller(self, controller):
        """Receive and process updated from target controller."""

        receiver = await controller.create_receiver()

        while self.running:
            for update in await receiver():
                await self.shedule_update_process(controller.TYPE, update)

            await asyncio.sleep(0.05)

    def run(self):
        """Start engine."""

        asyncio.set_event_loop(self.loop)

        self.loops = []

        for controller in self.controllers:
            self.loops.append(self.loop_for_controller(controller))

            awaitables = self.loop.run_until_complete(
                controller.create_tasks(self.ensure_future)
            )

            for awaitable in awaitables:
                self.loops.append(awaitable())

        self.loop.run_until_complete(self.shedule_update_process(
            "kutana",
            {
                "kutana": self, "update_type": "startup",
                "callbacks_owners": self.executor.callbacks_owners
            }
        ))

        try:
            self.gathered = asyncio.gather(*self.loops, *self.tasks)

            self.loop.run_until_complete(self.gathered)
        except (KeyboardInterrupt, ExitException):
            pass

        self.loop.run_until_complete(self.dispose())

    async def dispose(self):
        """Free resourses and prepare for shutdown."""

        await self.executor.dispose()

        for controller in self.controllers:
            await controller.dispose()
